/* eslint class-methods-use-this: 0 */
const _ = require('lodash');
const { Op } = require('sequelize');

const FIELDS_FOR_SORT = ['id', 'createdAt', 'updatedAt'];
const SORT_TYPES = ['DESC', 'ASC'];

class AuditQueryBuilder {
    constructor() {
        this.buildQuery = this.buildQuery.bind(this);
    }

    _buildFilters({ qs, accountId }) {
        return {
            accountId,
            ...this._buildSearchFilters(qs),
            ...this._buildDates(qs),
        };
    }

    _buildSearchFilters(qs) {
        const searchFilters = {};

        ['userName', 'userId', 'entity', 'entityId', 'action', 'rs_status'].forEach((fieldName) => {
            const fieldVal = _.get(qs, fieldName);

            if (_.isArray(fieldVal)) {
                searchFilters[fieldName] = { [Op.or]: fieldVal };
            } else if (fieldVal) {
                searchFilters[fieldName] = fieldVal;
            }
        });

        return searchFilters;
    }

    _buildPagination(qs) {
        let pageSize = Number(_.get(qs, 'pageSize'));
        if (isNaN(pageSize) || !_.inRange(pageSize, 1, 1000)) { // eslint-disable-line
            pageSize = 10;
        }

        let page = Number(_.get(qs, 'page'));
        if (isNaN(page) || page < 0) { // eslint-disable-line
            page = 0;
        }

        return {
            limit: pageSize,
            offset: pageSize * page,
        };
    }

    _buildSorting(qs) {
        const sortField = _.get(qs, 'sortField');
        const sortType = SORT_TYPES.includes(_.get(qs, 'sortType')) ? _.get(qs, 'sortType', '') : SORT_TYPES[0];

        if (!FIELDS_FOR_SORT.includes(sortField)) {
            return {};
        }

        return {
            order: [
                [sortField, sortType],
            ],
        };
    }

    _buildDates(qs) {
        const result = {};
        const fromDate = Number(_.get(qs, 'from'));
        const toDate = Number(_.get(qs, 'to'));

        if (isNaN(fromDate) && isNaN(toDate)) { // eslint-disable-line
            return {};
        }

        if (fromDate) {
            result[Op.gte] = fromDate;
        }

        if (toDate) {
            result[Op.lte] = toDate;
        }

        return {
            createdAt: result
        };
    }

    buildQuery(qs, accountId) {
        return {
            where: {
                ...this._buildFilters({ qs, accountId }),
            },
            ...this._buildSorting(qs),
            ...this._buildPagination(qs),
        };
    }
}


module.exports = new AuditQueryBuilder();
